@@ -18,16 +18,26 @@ use std::ops::RangeInclusive;
 use crate::*;
 
 #[derive(Clone, Copy, Debug)]
-#[cfg_attr(feature = "persistence", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 struct PanelState {
     rect: Rect,
 }
 
+impl PanelState {
+    fn load(ctx: &Context, bar_id: Id) -> Option<Self> {
+        ctx.memory().data.get_persisted(bar_id)
+    }
+
+    fn store(self, ctx: &Context, bar_id: Id) {
+        ctx.memory().data.insert_persisted(bar_id, self);
+    }
+}
+
 // ----------------------------------------------------------------------------
 
 /// `Left` or `Right`
 #[derive(Clone, Copy, Debug, PartialEq)]
-enum Side {
+pub enum Side {
     Left,
     Right,
 }
@@ -63,12 +73,11 @@ impl Side {
 /// See the [module level docs](crate::containers::panel) for more details.
 ///
 /// ```
-/// # let mut ctx = egui::CtxRef::default();
-/// # ctx.begin_frame(Default::default());
-/// # let ctx = &ctx;
+/// # egui::__run_test_ctx(|ctx| {
 /// egui::SidePanel::left("my_left_panel").show(ctx, |ui| {
 ///    ui.label("Hello World!");
 /// });
+/// # });
 /// ```
 ///
 /// See also [`TopBottomPanel`].
@@ -94,7 +103,7 @@ impl SidePanel {
     }
 
     /// `id_source`: Something unique, e.g. `"my_panel"`.
-    fn new(side: Side, id_source: impl std::hash::Hash) -> Self {
+    pub fn new(side: Side, id_source: impl std::hash::Hash) -> Self {
         Self {
             side,
             id: Id::new(id_source),
@@ -105,7 +114,8 @@ impl SidePanel {
         }
     }
 
-    /// Switch resizable on/off.
+    /// Can panel be resized by dragging the edge of it?
+    ///
     /// Default is `true`.
     pub fn resizable(mut self, resizable: bool) -> Self {
         self.resizable = resizable;
@@ -147,6 +157,15 @@ impl SidePanel {
         self,
         ui: &mut Ui,
         add_contents: impl FnOnce(&mut Ui) -> R,
+    ) -> InnerResponse<R> {
+        self.show_inside_dyn(ui, Box::new(add_contents))
+    }
+
+    /// Show the panel inside a `Ui`.
+    fn show_inside_dyn<'c, R>(
+        self,
+        ui: &mut Ui,
+        add_contents: Box<dyn FnOnce(&mut Ui) -> R + 'c>,
     ) -> InnerResponse<R> {
         let Self {
             side,
@@ -161,7 +180,7 @@ impl SidePanel {
         let mut panel_rect = available_rect;
         {
             let mut width = default_width;
-            if let Some(state) = ui.memory().id_data.get::<PanelState>(&id) {
+            if let Some(state) = PanelState::load(ui.ctx(), id) {
                 width = state.rect.width();
             }
             width = clamp_to_range(width, width_range.clone()).at_most(available_rect.width());
@@ -212,7 +231,7 @@ impl SidePanel {
         panel_ui.expand_to_include_rect(panel_rect);
         let frame = frame.unwrap_or_else(|| Frame::side_top_panel(ui.style()));
         let inner_response = frame.show(&mut panel_ui, |ui| {
-            ui.set_min_height(ui.max_rect_finite().height()); // Make sure the frame fills the full height
+            ui.set_min_height(ui.max_rect().height()); // Make sure the frame fills the full height
             ui.set_min_width(*width_range.start());
             add_contents(ui)
         });
@@ -233,7 +252,7 @@ impl SidePanel {
         }
         ui.expand_to_include_rect(rect);
 
-        ui.memory().id_data.insert(id, PanelState { rect });
+        PanelState { rect }.store(ui.ctx(), id);
 
         if resize_hover || is_resizing {
             let stroke = if is_resizing {
@@ -259,6 +278,15 @@ impl SidePanel {
         self,
         ctx: &CtxRef,
         add_contents: impl FnOnce(&mut Ui) -> R,
+    ) -> InnerResponse<R> {
+        self.show_dyn(ctx, Box::new(add_contents))
+    }
+
+    /// Show the panel at the top level.
+    fn show_dyn<'c, R>(
+        self,
+        ctx: &CtxRef,
+        add_contents: Box<dyn FnOnce(&mut Ui) -> R + 'c>,
     ) -> InnerResponse<R> {
         let layer_id = LayerId::background();
         let side = self.side;
@@ -266,7 +294,7 @@ impl SidePanel {
         let clip_rect = ctx.input().screen_rect();
         let mut panel_ui = Ui::new(ctx.clone(), layer_id, self.id, available_rect, clip_rect);
 
-        let inner_response = self.show_inside(&mut panel_ui, add_contents);
+        let inner_response = self.show_inside_dyn(&mut panel_ui, add_contents);
         let rect = inner_response.response.rect;
 
         match side {
@@ -285,7 +313,7 @@ impl SidePanel {
 
 /// `Top` or `Bottom`
 #[derive(Clone, Copy, Debug, PartialEq)]
-enum TopBottomSide {
+pub enum TopBottomSide {
     Top,
     Bottom,
 }
@@ -321,12 +349,11 @@ impl TopBottomSide {
 /// See the [module level docs](crate::containers::panel) for more details.
 ///
 /// ```
-/// # let mut ctx = egui::CtxRef::default();
-/// # ctx.begin_frame(Default::default());
-/// # let ctx = &ctx;
+/// # egui::__run_test_ctx(|ctx| {
 /// egui::TopBottomPanel::top("my_panel").show(ctx, |ui| {
 ///    ui.label("Hello World!");
 /// });
+/// # });
 /// ```
 ///
 /// See also [`SidePanel`].
@@ -352,7 +379,7 @@ impl TopBottomPanel {
     }
 
     /// `id_source`: Something unique, e.g. `"my_panel"`.
-    fn new(side: TopBottomSide, id_source: impl std::hash::Hash) -> Self {
+    pub fn new(side: TopBottomSide, id_source: impl std::hash::Hash) -> Self {
         Self {
             side,
             id: Id::new(id_source),
@@ -363,7 +390,8 @@ impl TopBottomPanel {
         }
     }
 
-    /// Switch resizable on/off.
+    /// Can panel be resized by dragging the edge of it?
+    ///
     /// Default is `false`.
     pub fn resizable(mut self, resizable: bool) -> Self {
         self.resizable = resizable;
@@ -406,6 +434,15 @@ impl TopBottomPanel {
         self,
         ui: &mut Ui,
         add_contents: impl FnOnce(&mut Ui) -> R,
+    ) -> InnerResponse<R> {
+        self.show_inside_dyn(ui, Box::new(add_contents))
+    }
+
+    /// Show the panel inside a `Ui`.
+    fn show_inside_dyn<'c, R>(
+        self,
+        ui: &mut Ui,
+        add_contents: Box<dyn FnOnce(&mut Ui) -> R + 'c>,
     ) -> InnerResponse<R> {
         let Self {
             side,
@@ -419,8 +456,7 @@ impl TopBottomPanel {
         let available_rect = ui.available_rect_before_wrap();
         let mut panel_rect = available_rect;
         {
-            let state = ui.memory().id_data.get::<PanelState>(&id).copied();
-            let mut height = if let Some(state) = state {
+            let mut height = if let Some(state) = PanelState::load(ui.ctx(), id) {
                 state.rect.height()
             } else {
                 default_height.unwrap_or_else(|| ui.style().spacing.interact_size.y)
@@ -473,7 +509,7 @@ impl TopBottomPanel {
         panel_ui.expand_to_include_rect(panel_rect);
         let frame = frame.unwrap_or_else(|| Frame::side_top_panel(ui.style()));
         let inner_response = frame.show(&mut panel_ui, |ui| {
-            ui.set_min_width(ui.max_rect_finite().width()); // Make the frame fill full width
+            ui.set_min_width(ui.max_rect().width()); // Make the frame fill full width
             ui.set_min_height(*height_range.start());
             add_contents(ui)
         });
@@ -494,7 +530,7 @@ impl TopBottomPanel {
         }
         ui.expand_to_include_rect(rect);
 
-        ui.memory().id_data.insert(id, PanelState { rect });
+        PanelState { rect }.store(ui.ctx(), id);
 
         if resize_hover || is_resizing {
             let stroke = if is_resizing {
@@ -520,6 +556,15 @@ impl TopBottomPanel {
         self,
         ctx: &CtxRef,
         add_contents: impl FnOnce(&mut Ui) -> R,
+    ) -> InnerResponse<R> {
+        self.show_dyn(ctx, Box::new(add_contents))
+    }
+
+    /// Show the panel at the top level.
+    fn show_dyn<'c, R>(
+        self,
+        ctx: &CtxRef,
+        add_contents: Box<dyn FnOnce(&mut Ui) -> R + 'c>,
     ) -> InnerResponse<R> {
         let layer_id = LayerId::background();
         let available_rect = ctx.available_rect();
@@ -528,7 +573,7 @@ impl TopBottomPanel {
         let clip_rect = ctx.input().screen_rect();
         let mut panel_ui = Ui::new(ctx.clone(), layer_id, self.id, available_rect, clip_rect);
 
-        let inner_response = self.show_inside(&mut panel_ui, add_contents);
+        let inner_response = self.show_inside_dyn(&mut panel_ui, add_contents);
         let rect = inner_response.response.rect;
 
         match side {
@@ -548,19 +593,6 @@ impl TopBottomPanel {
 
 // ----------------------------------------------------------------------------
 
-#[deprecated = "Use TopBottomPanel::top instead"]
-pub struct TopPanel {}
-
-#[allow(deprecated)]
-impl TopPanel {
-    #[deprecated = "Use TopBottomPanel::top instead"]
-    pub fn top(id_source: impl std::hash::Hash) -> TopBottomPanel {
-        TopBottomPanel::top(id_source)
-    }
-}
-
-// ----------------------------------------------------------------------------
-
 /// A panel that covers the remainder of the screen,
 /// i.e. whatever area is left after adding other panels.
 ///
@@ -571,12 +603,11 @@ impl TopPanel {
 /// See the [module level docs](crate::containers::panel) for more details.
 ///
 /// ```
-/// # let mut ctx = egui::CtxRef::default();
-/// # ctx.begin_frame(Default::default());
-/// # let ctx = &ctx;
+/// # egui::__run_test_ctx(|ctx| {
 /// egui::CentralPanel::default().show(ctx, |ui| {
 ///    ui.label("Hello World!");
 /// });
+/// # });
 /// ```
 #[must_use = "You should call .show()"]
 #[derive(Default)]
@@ -598,10 +629,19 @@ impl CentralPanel {
         self,
         ui: &mut Ui,
         add_contents: impl FnOnce(&mut Ui) -> R,
+    ) -> InnerResponse<R> {
+        self.show_inside_dyn(ui, Box::new(add_contents))
+    }
+
+    /// Show the panel inside a `Ui`.
+    fn show_inside_dyn<'c, R>(
+        self,
+        ui: &mut Ui,
+        add_contents: Box<dyn FnOnce(&mut Ui) -> R + 'c>,
     ) -> InnerResponse<R> {
         let Self { frame } = self;
 
-        let panel_rect = ui.available_rect_before_wrap_finite();
+        let panel_rect = ui.available_rect_before_wrap();
         let mut panel_ui = ui.child_ui(panel_rect, Layout::top_down(Align::Min));
 
         let frame = frame.unwrap_or_else(|| Frame::central_panel(ui.style()));
@@ -616,6 +656,15 @@ impl CentralPanel {
         self,
         ctx: &CtxRef,
         add_contents: impl FnOnce(&mut Ui) -> R,
+    ) -> InnerResponse<R> {
+        self.show_dyn(ctx, Box::new(add_contents))
+    }
+
+    /// Show the panel at the top level.
+    fn show_dyn<'c, R>(
+        self,
+        ctx: &CtxRef,
+        add_contents: Box<dyn FnOnce(&mut Ui) -> R + 'c>,
     ) -> InnerResponse<R> {
         let available_rect = ctx.available_rect();
         let layer_id = LayerId::background();
@@ -624,7 +673,7 @@ impl CentralPanel {
         let clip_rect = ctx.input().screen_rect();
         let mut panel_ui = Ui::new(ctx.clone(), layer_id, id, available_rect, clip_rect);
 
-        let inner_response = self.show_inside(&mut panel_ui, add_contents);
+        let inner_response = self.show_inside_dyn(&mut panel_ui, add_contents);
 
         // Only inform ctx about what we actually used, so we can shrink the native window to fit.
         ctx.frame_state()
