@@ -11,6 +11,7 @@ use crate::{
     frame_state::FrameState,
     input_state::*,
     layers::GraphicLayers,
+    menu::ContextMenuSystem,
     mutex::{Mutex, MutexGuard},
     *,
 };
@@ -25,6 +26,9 @@ use epaint::{stats::*, text::Fonts, *};
 ///
 /// [`CtxRef`] is cheap to clone, and any clones refers to the same mutable data.
 ///
+/// A [`CtxRef`] is only valid for the duration of a frame, and so you should not store a [`CtxRef`] between frames.
+/// A new [`CtxRef`] is created each frame by calling [`Self::run`].
+///
 /// # Example:
 ///
 /// ``` no_run
@@ -35,22 +39,19 @@ use epaint::{stats::*, text::Fonts, *};
 /// // Game loop:
 /// loop {
 ///     let raw_input = egui::RawInput::default();
-///     ctx.begin_frame(raw_input);
-///
-///     egui::CentralPanel::default().show(&ctx, |ui| {
-///         ui.label("Hello world!");
-///         if ui.button("Click me").clicked() {
-///             /* take some action here */
-///         }
+///     let (output, shapes) = ctx.run(raw_input, |ctx| {
+///         egui::CentralPanel::default().show(&ctx, |ui| {
+///             ui.label("Hello world!");
+///             if ui.button("Click me").clicked() {
+///                 // take some action here
+///             }
+///         });
 ///     });
-///
-///     let (output, shapes) = ctx.end_frame();
 ///     let clipped_meshes = ctx.tessellate(shapes); // create triangles to paint
 ///     handle_output(output);
 ///     paint(clipped_meshes);
 /// }
 /// ```
-///
 #[derive(Clone)]
 pub struct CtxRef(std::sync::Arc<Context>);
 
@@ -92,12 +93,56 @@ impl Default for CtxRef {
 }
 
 impl CtxRef {
-    /// Call at the start of every frame. Match with a call to [`Context::end_frame`].
+    /// Run the ui code for one frame.
+    ///
+    /// Put your widgets into a [`SidePanel`], [`TopBottomPanel`], [`CentralPanel`], [`Window`] or [`Area`].
     ///
     /// This will modify the internal reference to point to a new generation of [`Context`].
     /// Any old clones of this [`CtxRef`] will refer to the old [`Context`], which will not get new input.
     ///
-    /// Put your widgets into a [`SidePanel`], [`TopBottomPanel`], [`CentralPanel`], [`Window`] or [`Area`].
+    /// You can alternatively run [`Self::begin_single_pass_frame`] and [`Self::end_single_pass_frame`].
+    ///
+    /// ``` rust
+    /// // One egui context that you keep reusing:
+    /// let mut ctx = egui::CtxRef::default();
+    ///
+    /// // Each frame:
+    /// let input = egui::RawInput::default();
+    /// let (output, shapes) = ctx.run(input, |ctx| {
+    ///     egui::CentralPanel::default().show(&ctx, |ui| {
+    ///         ui.label("Hello egui!");
+    ///     });
+    /// });
+    /// // handle output, paint shapes
+    /// ```
+    #[must_use]
+    pub fn run(
+        &mut self,
+        new_input: RawInput,
+        run_ui: impl FnOnce(&CtxRef),
+    ) -> (Output, Vec<ClippedShape>) {
+        self.begin_frame(new_input);
+        run_ui(self);
+        self.end_frame()
+    }
+
+    /// An alternative to calling [`Self::run`].
+    ///
+    /// ``` rust
+    /// // One egui context that you keep reusing:
+    /// let mut ctx = egui::CtxRef::default();
+    ///
+    /// // Each frame:
+    /// let input = egui::RawInput::default();
+    /// ctx.begin_frame(input);
+    ///
+    /// egui::CentralPanel::default().show(&ctx, |ui| {
+    ///     ui.label("Hello egui!");
+    /// });
+    ///
+    /// let (output, shapes) = ctx.end_frame();
+    /// // handle output, paint shapes
+    /// ```
     pub fn begin_frame(&mut self, new_input: RawInput) {
         let mut self_: Context = (*self.0).clone();
         self_.begin_frame_mut(new_input);
@@ -199,22 +244,24 @@ impl CtxRef {
             changed: false, // must be set by the widget itself
         };
 
+        let mut memory = self.memory();
+
         if !enabled || !sense.focusable || !layer_id.allow_interaction() {
             // Not interested or allowed input:
-            self.memory().surrender_focus(id);
+            memory.surrender_focus(id);
             return response;
         }
 
         // We only want to focus labels if the screen reader is on.
         let interested_in_focus =
-            sense.interactive() || sense.focusable && self.memory().options.screen_reader;
+            sense.interactive() || sense.focusable && memory.options.screen_reader;
 
         if interested_in_focus {
-            self.memory().interested_in_focus(id);
+            memory.interested_in_focus(id);
         }
 
         if sense.click
-            && response.has_focus()
+            && memory.has_focus(response.id)
             && (self.input().key_pressed(Key::Space) || self.input().key_pressed(Key::Enter))
         {
             // Space/enter works like a primary click for e.g. selected buttons
@@ -224,8 +271,6 @@ impl CtxRef {
         self.register_interaction_id(id, rect);
 
         if sense.click || sense.drag {
-            let mut memory = self.memory();
-
             memory.interaction.click_interest |= hovered && sense.click;
             memory.interaction.drag_interest |= hovered && sense.drag;
 
@@ -287,8 +332,8 @@ impl CtxRef {
             response.hovered &= response.is_pointer_button_down_on; // we don't hover widgets while interacting with *other* widgets
         }
 
-        if response.has_focus() && response.clicked_elsewhere() {
-            self.memory().surrender_focus(id);
+        if memory.has_focus(response.id) && response.clicked_elsewhere() {
+            memory.surrender_focus(id);
         }
 
         response
@@ -307,16 +352,17 @@ impl CtxRef {
 
 // ----------------------------------------------------------------------------
 
-/// This is the first thing you need when working with egui. Create using [`CtxRef`].
+/// Your handle to egui.
+///
+/// This is the first thing you need when working with egui.
+/// Use [`CtxRef`] to create and refer to a [`Context`].
 ///
 /// Contains the [`InputState`], [`Memory`], [`Output`], and more.
 ///
-/// Your handle to Egui.
-///
-/// Almost all methods are marked `&self`, `Context` has interior mutability (protected by mutexes).
+/// Almost all methods are marked `&self`, [`Context`] has interior mutability (protected by mutexes).
 /// Multi-threaded access to a [`Context`] is behind the feature flag `multi_threaded`.
 /// Normally you'd always do all ui work on one thread, or perhaps use multiple contexts,
-/// but if you really want to access the same Context from multiple threads, it *SHOULD* be fine,
+/// but if you really want to access the same [`Context`] from multiple threads, it *SHOULD* be fine,
 /// but you are likely the first person to try it.
 #[derive(Default)]
 pub struct Context {
@@ -325,10 +371,11 @@ pub struct Context {
     // This means everything else needs to be behind an Arc.
     // We can probably come up with a nicer design.
     //
-    /// None until first call to `begin_frame`.
+    /// `None` until the start of the first frame.
     fonts: Option<Arc<Fonts>>,
     memory: Arc<Mutex<Memory>>,
     animation_manager: Arc<Mutex<AnimationManager>>,
+    context_menu_system: Arc<Mutex<ContextMenuSystem>>,
 
     input: InputState,
 
@@ -357,17 +404,12 @@ impl Clone for Context {
             output: self.output.clone(),
             paint_stats: self.paint_stats.clone(),
             repaint_requests: self.repaint_requests.load(SeqCst).into(),
+            context_menu_system: self.context_menu_system.clone(),
         }
     }
 }
 
 impl Context {
-    #[allow(clippy::new_ret_no_self)]
-    #[deprecated = "Use CtxRef::default() instead"]
-    pub fn new() -> CtxRef {
-        CtxRef::default()
-    }
-
     /// How much space is still available after panels has been added.
     /// This is the "background" area, what egui doesn't cover with panels (but may cover with windows).
     /// This is also the area to which windows are constrained.
@@ -381,6 +423,10 @@ impl Context {
         self.memory.lock()
     }
 
+    pub(crate) fn context_menu_system(&self) -> MutexGuard<'_, ContextMenuSystem> {
+        self.context_menu_system.lock()
+    }
+
     pub(crate) fn graphics(&self) -> MutexGuard<'_, GraphicLayers> {
         self.graphics.lock()
     }
@@ -408,20 +454,26 @@ impl Context {
         &self.input
     }
 
-    /// Not valid until first call to [`CtxRef::begin_frame()`].
+    /// Not valid until first call to [`CtxRef::run()`].
     /// That's because since we don't know the proper `pixels_per_point` until then.
     pub fn fonts(&self) -> &Fonts {
         &*self
             .fonts
             .as_ref()
-            .expect("No fonts available until first call to CtxRef::begin_frame()")
+            .expect("No fonts available until first call to CtxRef::run()")
     }
 
-    /// The egui texture, containing font characters etc.
-    /// Not valid until first call to [`CtxRef::begin_frame()`].
+    /// The egui font image, containing font characters etc.
+    ///
+    /// Not valid until first call to [`CtxRef::run()`].
     /// That's because since we don't know the proper `pixels_per_point` until then.
-    pub fn texture(&self) -> Arc<epaint::Texture> {
-        self.fonts().texture()
+    pub fn font_image(&self) -> Arc<epaint::FontImage> {
+        self.fonts().font_image()
+    }
+
+    #[deprecated = "Renamed font_image"]
+    pub fn texture(&self) -> Arc<epaint::FontImage> {
+        self.fonts().font_image()
     }
 
     /// Tell `egui` which fonts to use.
@@ -486,6 +538,10 @@ impl Context {
     /// Note that this may be overwritten by input from the integration via [`RawInput::pixels_per_point`].
     /// For instance, when using `egui_web` the browsers native zoom level will always be used.
     pub fn set_pixels_per_point(&self, pixels_per_point: f32) {
+        if pixels_per_point != self.pixels_per_point() {
+            self.request_repaint();
+        }
+
         self.memory().new_pixels_per_point = Some(pixels_per_point);
     }
 
@@ -515,15 +571,12 @@ impl Context {
 
     // ---------------------------------------------------------------------
 
-    /// Constrain the position of a window/area
-    /// so it fits within the screen.
-    pub(crate) fn constrain_window_rect(&self, window: Rect) -> Rect {
-        self.constrain_window_rect_to_area(window, self.available_rect())
-    }
+    /// Constrain the position of a window/area so it fits within the provided boundary.
+    ///
+    /// If area is `None`, will constrain to [`Self::available_rect`].
+    pub(crate) fn constrain_window_rect_to_area(&self, window: Rect, area: Option<Rect>) -> Rect {
+        let mut area = area.unwrap_or_else(|| self.available_rect());
 
-    /// Constrain the position of a window/area
-    /// so it fits within the provided boundary.
-    pub(crate) fn constrain_window_rect_to_area(&self, window: Rect, mut area: Rect) -> Rect {
         if window.width() > area.width() {
             // Allow overlapping side bars.
             // This is important for small screens, e.g. mobiles running the web demo.
@@ -565,30 +618,7 @@ impl Context {
         self.input = input.begin_frame(new_raw_input);
         self.frame_state.lock().begin_frame(&self.input);
 
-        {
-            // Load new fonts if required:
-            let new_font_definitions = self.memory().new_font_definitions.take();
-            let pixels_per_point = self.input.pixels_per_point();
-
-            let pixels_per_point_changed = match &self.fonts {
-                None => true,
-                Some(current_fonts) => {
-                    (current_fonts.pixels_per_point() - pixels_per_point).abs() > 1e-3
-                }
-            };
-
-            if self.fonts.is_none() || new_font_definitions.is_some() || pixels_per_point_changed {
-                self.fonts = Some(Arc::new(Fonts::from_definitions(
-                    pixels_per_point,
-                    new_font_definitions.unwrap_or_else(|| {
-                        self.fonts
-                            .as_ref()
-                            .map(|font| font.definitions().clone())
-                            .unwrap_or_default()
-                    }),
-                )));
-            }
-        }
+        self.update_fonts(self.input.pixels_per_point());
 
         // Ensure we register the background area so panels and background ui can catch clicks:
         let screen_rect = self.input.screen_rect();
@@ -602,6 +632,30 @@ impl Context {
         );
     }
 
+    /// Load fonts unless already loaded.
+    fn update_fonts(&mut self, pixels_per_point: f32) {
+        let new_font_definitions = self.memory().new_font_definitions.take();
+
+        let pixels_per_point_changed = match &self.fonts {
+            None => true,
+            Some(current_fonts) => {
+                (current_fonts.pixels_per_point() - pixels_per_point).abs() > 1e-3
+            }
+        };
+
+        if self.fonts.is_none() || new_font_definitions.is_some() || pixels_per_point_changed {
+            self.fonts = Some(Arc::new(Fonts::new(
+                pixels_per_point,
+                new_font_definitions.unwrap_or_else(|| {
+                    self.fonts
+                        .as_ref()
+                        .map(|font| font.definitions().clone())
+                        .unwrap_or_default()
+                }),
+            )));
+        }
+    }
+
     /// Call at the end of each frame.
     /// Returns what has happened this frame [`crate::Output`] as well as what you need to paint.
     /// You can transform the returned shapes into triangles with a call to [`Context::tessellate`].
@@ -633,14 +687,18 @@ impl Context {
 
     /// Tessellate the given shapes into triangle meshes.
     pub fn tessellate(&self, shapes: Vec<ClippedShape>) -> Vec<ClippedMesh> {
+        // A tempting optimization is to reuse the tessellation from last frame if the
+        // shapes are the same, but just comparing the shapes takes about 50% of the time
+        // it takes to tessellate them, so it is not a worth optimization.
+
         let mut tessellation_options = self.memory().options.tessellation_options;
         tessellation_options.pixels_per_point = self.pixels_per_point();
         tessellation_options.aa_size = 1.0 / self.pixels_per_point();
-        let paint_stats = PaintStats::from_shapes(&shapes); // TODO: internal allocations
+        let paint_stats = PaintStats::from_shapes(&shapes);
         let clipped_meshes = tessellator::tessellate_shapes(
             shapes,
             tessellation_options,
-            self.fonts().texture().size(),
+            self.fonts().font_image().size(),
         );
         *self.paint_stats.lock() = paint_stats.with_clipped_meshes(&clipped_meshes);
         clipped_meshes
@@ -697,16 +755,6 @@ impl Context {
         self.memory().interaction.is_using_pointer()
     }
 
-    #[deprecated = "Renamed wants_pointer_input"]
-    pub fn wants_mouse_input(&self) -> bool {
-        self.wants_pointer_input()
-    }
-
-    #[deprecated = "Renamed is_using_pointer"]
-    pub fn is_using_mouse(&self) -> bool {
-        self.is_using_pointer()
-    }
-
     /// If `true`, egui is currently listening on text input (e.g. typing text in a [`TextEdit`]).
     pub fn wants_keyboard_input(&self) -> bool {
         self.memory().interaction.focus.focused().is_some()
@@ -760,8 +808,15 @@ impl Context {
     /// Calling this with `value = false` will always yield a number less than one, quickly going towards zero.
     ///
     /// The function will call [`Self::request_repaint()`] when appropriate.
+    ///
+    /// The animation time is taken from [`Style::animation_time`].
     pub fn animate_bool(&self, id: Id, value: bool) -> f32 {
         let animation_time = self.style().animation_time;
+        self.animate_bool_with_time(id, value, animation_time)
+    }
+
+    /// Like [`Self::animate_bool`] but allows you to control the animation time.
+    pub fn animate_bool_with_time(&self, id: Id, value: bool, animation_time: f32) -> f32 {
         let animated_value =
             self.animation_manager
                 .lock()
@@ -794,7 +849,7 @@ impl Context {
             .show(ui, |ui| {
                 let mut font_definitions = self.fonts().definitions().clone();
                 font_definitions.ui(ui);
-                self.fonts().texture().ui(ui);
+                self.fonts().font_image().ui(ui);
                 self.set_fonts(font_definitions);
             });
 
@@ -879,6 +934,13 @@ impl Context {
             *self.memory() = Default::default();
         }
 
+        let num_state = self.memory().data.len();
+        let num_serialized = self.memory().data.count_serialized();
+        ui.label(format!(
+            "{} widget states stored (of which {} are serialized).",
+            num_state, num_serialized
+        ));
+
         ui.horizontal(|ui| {
             ui.label(format!(
                 "{} areas (panels, windows, popups, …)",
@@ -902,7 +964,7 @@ impl Context {
                     let text = format!("{} - {:?}", layer_id.short_debug_format(), area.rect(),);
                     // TODO: `Sense::hover_highlight()`
                     if ui
-                        .add(Label::new(text).monospace().sense(Sense::click()))
+                        .add(Label::new(RichText::new(text).monospace()).sense(Sense::click()))
                         .hovered
                         && is_visible
                     {
@@ -918,12 +980,12 @@ impl Context {
             ui.label(format!(
                 "{} collapsing headers",
                 self.memory()
-                    .id_data
+                    .data
                     .count::<containers::collapsing_header::State>()
             ));
             if ui.button("Reset").clicked() {
                 self.memory()
-                    .id_data
+                    .data
                     .remove_by_type::<containers::collapsing_header::State>();
             }
         });
@@ -931,32 +993,30 @@ impl Context {
         ui.horizontal(|ui| {
             ui.label(format!(
                 "{} menu bars",
-                self.memory().id_data_temp.count::<menu::BarState>()
+                self.memory().data.count::<menu::BarState>()
             ));
             if ui.button("Reset").clicked() {
-                self.memory()
-                    .id_data_temp
-                    .remove_by_type::<menu::BarState>();
+                self.memory().data.remove_by_type::<menu::BarState>();
             }
         });
 
         ui.horizontal(|ui| {
             ui.label(format!(
                 "{} scroll areas",
-                self.memory().id_data.count::<scroll_area::State>()
+                self.memory().data.count::<scroll_area::State>()
             ));
             if ui.button("Reset").clicked() {
-                self.memory().id_data.remove_by_type::<scroll_area::State>();
+                self.memory().data.remove_by_type::<scroll_area::State>();
             }
         });
 
         ui.horizontal(|ui| {
             ui.label(format!(
                 "{} resize areas",
-                self.memory().id_data.count::<resize::State>()
+                self.memory().data.count::<resize::State>()
             ));
             if ui.button("Reset").clicked() {
-                self.memory().id_data.remove_by_type::<resize::State>();
+                self.memory().data.remove_by_type::<resize::State>();
             }
         });
 
