@@ -3,9 +3,9 @@
 //! Try the live web demo: <https://emilk.github.io/egui/index.html>. Read more about egui at <https://github.com/emilk/egui>.
 //!
 //! `egui` is in heavy development, with each new version having breaking changes.
-//! You need to have rust 1.51.0 or later to use `egui`.
+//! You need to have rust 1.56.0 or later to use `egui`.
 //!
-//! To quickly get started with egui, you can take a look at [`egui_template`](https://github.com/emilk/egui_template)
+//! To quickly get started with egui, you can take a look at [`eframe_template`](https://github.com/emilk/eframe_template)
 //! which uses [`eframe`](https://docs.rs/eframe).
 //!
 //! To create a GUI using egui you first need a [`CtxRef`] (by convention referred to by `ctx`).
@@ -16,7 +16,7 @@
 //!
 //! To see what is possible to build with egui you can check out the online demo at <https://emilk.github.io/egui/#demo>.
 //!
-//! If you like the "learning by doing" approach, clone <https://github.com/emilk/egui_template> and get started using egui right away.
+//! If you like the "learning by doing" approach, clone <https://github.com/emilk/eframe_template> and get started using egui right away.
 //!
 //! ### A simple example
 //!
@@ -45,21 +45,21 @@
 //! get access to an [`Ui`] where you can put widgets. For example:
 //!
 //! ```
-//! # let mut ctx = egui::CtxRef::default();
-//! # ctx.begin_frame(Default::default());
+//! # egui::__run_test_ctx(|ctx| {
 //! egui::CentralPanel::default().show(&ctx, |ui| {
 //!     ui.add(egui::Label::new("Hello World!"));
 //!     ui.label("A shorter and more convenient way to add a label.");
 //!     if ui.button("Click me").clicked() {
-//!         /* take some action here */
+//!         // take some action here
 //!     }
 //! });
+//! # });
 //! ```
 //!
 //! ### Quick start
 //!
 //! ``` rust
-//! # let ui = &mut egui::Ui::__test();
+//! # egui::__run_test_ui(|ui| {
 //! # let mut my_string = String::new();
 //! # let mut my_boolean = true;
 //! # let mut my_f32 = 42.0;
@@ -89,6 +89,7 @@
 //! ui.collapsing("Click to see what is hidden!", |ui| {
 //!     ui.label("Not much, as it turns out");
 //! });
+//! # });
 //! ```
 //!
 //! ## Conventions
@@ -117,16 +118,16 @@
 //! // Game loop:
 //! loop {
 //!     let raw_input: egui::RawInput = gather_input();
-//!     ctx.begin_frame(raw_input);
 //!
-//!     egui::CentralPanel::default().show(&ctx, |ui| {
-//!         ui.label("Hello world!");
-//!         if ui.button("Click me").clicked() {
-//!             /* take some action here */
-//!         }
+//!     let (output, shapes) = ctx.run(raw_input, |ctx| {
+//!         egui::CentralPanel::default().show(&ctx, |ui| {
+//!             ui.label("Hello world!");
+//!             if ui.button("Click me").clicked() {
+//!                 // take some action here
+//!             }
+//!         });
 //!     });
 //!
-//!     let (output, shapes) = ctx.end_frame();
 //!     let clipped_meshes = ctx.tessellate(shapes); // create triangles to paint
 //!     handle_output(output);
 //!     paint(clipped_meshes);
@@ -141,10 +142,11 @@
 //! Here is an example to illustrate it:
 //!
 //! ```
-//! # let ui = &mut egui::Ui::__test();
+//! # egui::__run_test_ui(|ui| {
 //! if ui.button("click me").clicked() {
 //!     take_action()
 //! }
+//! # });
 //! # fn take_action() {}
 //! ```
 //!
@@ -166,17 +168,19 @@
 //! ## How widgets works
 //!
 //! ```
-//! # let ui = &mut egui::Ui::__test();
+//! # egui::__run_test_ui(|ui| {
 //! if ui.button("click me").clicked() { take_action() }
+//! # });
 //! # fn take_action() {}
 //! ```
 //!
 //! is short for
 //!
 //! ```
-//! # let ui = &mut egui::Ui::__test();
+//! # egui::__run_test_ui(|ui| {
 //! let button = egui::Button::new("click me");
 //! if ui.add(button).clicked() { take_action() }
+//! # });
 //! # fn take_action() {}
 //! ```
 //!
@@ -184,10 +188,11 @@
 //!
 //! ```
 //! # use egui::Widget;
-//! # let ui = &mut egui::Ui::__test();
+//! # egui::__run_test_ui(|ui| {
 //! let button = egui::Button::new("click me");
 //! let response = button.ui(ui);
 //! if response.clicked() { take_action() }
+//! # });
 //! # fn take_action() {}
 //! ```
 //!
@@ -202,53 +207,75 @@
 //! }
 //! ```
 //!
+//! ## Auto-sizing panels and windows
+//! In egui, all panels and windows auto-shrink to fit the content.
+//! If the window or panel is also resizable, this can lead to a weird behavior
+//! where you can drag the edge of the panel/window to make it larger, and
+//! when you release the panel/window shrinks again.
+//! This is an artifact of immediate mode, and here are some alternatives on how to avoid it:
+//!
+//! 1. Turn of resizing with [`Window::resizable`], [`SidePanel::resizable`], [`TopBottomPanel::resizable`].
+//! 2. Wrap your panel contents in a [`ScrollArea`], or use [`Window::vscroll`] and [`Window::hscroll`].
+//! 3. Use a justified layout:
+//!
+//! ``` rust
+//! # egui::__run_test_ui(|ui| {
+//! ui.with_layout(egui::Layout::top_down_justified(egui::Align::Center), |ui| {
+//!     ui.button("I am becoming wider as needed");
+//! });
+//! # });
+//! ```
+//!
+//! 4. Fill in extra space with emptiness:
+//!
+//! ``` rust
+//! # egui::__run_test_ui(|ui| {
+//! ui.allocate_space(ui.available_size()); // put this LAST in your panel/window code
+//! # });
+//! ```
+//!
 //! ## Sizes
 //! You can control the size of widgets using [`Ui::add_sized`].
 //!
 //! ```
-//! # let ui = &mut egui::Ui::__test();
+//! # egui::__run_test_ui(|ui| {
 //! # let mut my_value = 0.0_f32;
 //! ui.add_sized([40.0, 20.0], egui::DragValue::new(&mut my_value));
+//! # });
 //! ```
 //!
 //! ## Code snippets
 //!
 //! ```
-//! # let ui = &mut egui::Ui::__test();
+//! # egui::__run_test_ui(|ui| {
 //! # let mut some_bool = true;
 //! // Miscellaneous tips and tricks
 //!
-//! ui.horizontal_wrapped(|ui|{
+//! ui.horizontal_wrapped(|ui| {
 //!     ui.spacing_mut().item_spacing.x = 0.0; // remove spacing between widgets
 //!     // `radio_value` also works for enums, integers, and more.
 //!     ui.radio_value(&mut some_bool, false, "Off");
 //!     ui.radio_value(&mut some_bool, true, "On");
 //! });
 //!
-//! ui.group(|ui|{
+//! ui.group(|ui| {
 //!     ui.label("Within a frame");
 //!     ui.set_min_height(200.0);
 //! });
 //!
 //! // A `scope` creates a temporary [`Ui`] in which you can change settings:
-//! ui.scope(|ui|{
+//! ui.scope(|ui| {
 //!     ui.visuals_mut().override_text_color = Some(egui::Color32::RED);
 //!     ui.style_mut().override_text_style = Some(egui::TextStyle::Monospace);
 //!     ui.style_mut().wrap = Some(false);
 //!
 //!     ui.label("This text will be red, monospace, and won't wrap to a new line");
 //! }); // the temporary settings are reverted here
+//! # });
 //! ```
 
 // Forbid warnings in release builds:
 #![cfg_attr(not(debug_assertions), deny(warnings))]
-// Disabled so we can support rust 1.51:
-// #![deny(
-//     rustdoc::broken_intra_doc_links,
-//     rustdoc::invalid_codeblock_attributes,
-//     rustdoc::missing_crate_level_docs,
-//     rustdoc::private_intra_doc_links
-// )]
 #![forbid(unsafe_code)]
 #![warn(
     clippy::all,
@@ -257,6 +284,7 @@
     clippy::checked_conversions,
     clippy::dbg_macro,
     clippy::debug_assert_with_mut_call,
+    clippy::disallowed_method,
     clippy::doc_markdown,
     clippy::empty_enum,
     clippy::enum_glob_use,
@@ -266,12 +294,17 @@
     clippy::explicit_into_iter_loop,
     clippy::fallible_impl_from,
     clippy::filter_map_next,
+    clippy::flat_map_option,
     clippy::float_cmp_const,
     clippy::fn_params_excessive_bools,
+    clippy::from_iter_instead_of_collect,
     clippy::if_let_mutex,
+    clippy::implicit_clone,
     clippy::imprecise_flops,
     clippy::inefficient_to_string,
     clippy::invalid_upcast_comparisons,
+    clippy::large_digit_groups,
+    clippy::large_stack_arrays,
     clippy::large_types_passed_by_value,
     clippy::let_unit_value,
     clippy::linkedlist,
@@ -280,8 +313,10 @@
     clippy::manual_ok_or,
     clippy::map_err_ignore,
     clippy::map_flatten,
+    clippy::map_unwrap_or,
     clippy::match_on_vec_items,
     clippy::match_same_arms,
+    clippy::match_wild_err_arm,
     clippy::match_wildcard_for_single_variants,
     clippy::mem_forget,
     clippy::mismatched_target_os,
@@ -291,6 +326,7 @@
     clippy::mutex_integer,
     clippy::needless_borrow,
     clippy::needless_continue,
+    clippy::needless_for_each,
     clippy::needless_pass_by_value,
     clippy::option_option,
     clippy::path_buf_push_overwrite,
@@ -298,6 +334,8 @@
     clippy::ref_option_ref,
     clippy::rest_pat_in_fully_bound_structs,
     clippy::same_functions_in_if_condition,
+    clippy::semicolon_if_nothing_returned,
+    clippy::single_match_else,
     clippy::string_add_assign,
     clippy::string_add,
     clippy::string_lit_as_bytes,
@@ -312,13 +350,13 @@
     clippy::zero_sized_map_values,
     future_incompatible,
     nonstandard_style,
-    rust_2018_idioms
+    rust_2018_idioms,
+    rustdoc::missing_crate_level_docs
 )]
 #![allow(clippy::float_cmp)]
 #![allow(clippy::manual_range_contains)]
 
 mod animation_manager;
-pub mod any;
 pub mod containers;
 mod context;
 mod data;
@@ -338,24 +376,26 @@ mod sense;
 pub mod style;
 mod ui;
 pub mod util;
+mod widget_text;
 pub mod widgets;
 
 pub use epaint;
 pub use epaint::emath;
 
-// Can't add deprecation notice due to https://github.com/rust-lang/rust/issues/30827
-pub use epaint as paint; // historical reasons
-
-// Can't add deprecation notice due to https://github.com/rust-lang/rust/issues/30827
-pub use emath as math; // historical reasons
-
 pub use emath::{lerp, pos2, remap, remap_clamp, vec2, Align, Align2, NumExt, Pos2, Rect, Vec2};
 pub use epaint::{
     color, mutex,
-    text::{FontDefinitions, FontFamily, TextStyle},
-    ClippedMesh, Color32, Rgba, Shape, Stroke, Texture, TextureId,
+    text::{FontData, FontDefinitions, FontFamily, TextStyle},
+    ClippedMesh, Color32, FontImage, Rgba, Shape, Stroke, TextureId,
 };
 
+pub mod text {
+    pub use epaint::text::{
+        FontData, FontDefinitions, FontFamily, Fonts, Galley, LayoutJob, LayoutSection, TextFormat,
+        TextStyle, TAB_SIZE,
+    };
+}
+
 pub use {
     containers::*,
     context::{Context, CtxRef},
@@ -364,7 +404,7 @@ pub use {
         output::{self, CursorIcon, Output, WidgetInfo},
     },
     grid::Grid,
-    id::Id,
+    id::{Id, IdMap},
     input_state::{InputState, MultiTouchInfo, PointerState},
     layers::{LayerId, Order},
     layout::*,
@@ -373,7 +413,9 @@ pub use {
     response::{InnerResponse, Response},
     sense::Sense,
     style::{Style, Visuals},
+    text::{Galley, TextFormat},
     ui::Ui,
+    widget_text::{RichText, WidgetText},
     widgets::*,
 };
 
@@ -383,9 +425,9 @@ pub use {
 pub fn warn_if_debug_build(ui: &mut crate::Ui) {
     if cfg!(debug_assertions) {
         ui.label(
-            crate::Label::new("‼ Debug build ‼")
+            RichText::new("‼ Debug build ‼")
                 .small()
-                .text_color(crate::Color32::RED),
+                .color(crate::Color32::RED),
         )
         .on_hover_text("egui was compiled with debug assertions enabled.");
     }
@@ -396,28 +438,30 @@ pub fn warn_if_debug_build(ui: &mut crate::Ui) {
 /// Create a [`Hyperlink`](crate::Hyperlink) to the current [`file!()`] (and line) on Github
 ///
 /// ```
-/// # let ui = &mut egui::Ui::__test();
+/// # egui::__run_test_ui(|ui| {
 /// ui.add(egui::github_link_file_line!("https://github.com/YOUR/PROJECT/blob/master/", "(source code)"));
+/// # });
 /// ```
 #[macro_export]
 macro_rules! github_link_file_line {
-    ($github_url:expr, $label:expr) => {{
+    ($github_url: expr, $label: expr) => {{
         let url = format!("{}{}#L{}", $github_url, file!(), line!());
-        $crate::Hyperlink::new(url).text($label)
+        $crate::Hyperlink::from_label_and_url($label, url)
     }};
 }
 
 /// Create a [`Hyperlink`](crate::Hyperlink) to the current [`file!()`] on github.
 ///
 /// ```
-/// # let ui = &mut egui::Ui::__test();
+/// # egui::__run_test_ui(|ui| {
 /// ui.add(egui::github_link_file!("https://github.com/YOUR/PROJECT/blob/master/", "(source code)"));
+/// # });
 /// ```
 #[macro_export]
 macro_rules! github_link_file {
-    ($github_url:expr, $label:expr) => {{
+    ($github_url: expr, $label: expr) => {{
         let url = format!("{}{}", $github_url, file!());
-        $crate::Hyperlink::new(url).text($label)
+        $crate::Hyperlink::from_label_and_url($label, url)
     }};
 }
 
@@ -426,7 +470,7 @@ macro_rules! github_link_file {
 /// Show debug info on hover when [`Context::set_debug_on_hover`] has been turned on.
 ///
 /// ```
-/// # let ui = &mut egui::Ui::__test();
+/// # egui::__run_test_ui(|ui| {
 /// // Turn on tracing of widgets
 /// ui.ctx().set_debug_on_hover(true);
 ///
@@ -435,24 +479,25 @@ macro_rules! github_link_file {
 ///
 /// /// Show [`std::file`] and [`std::line`] on hover
 /// egui::trace!(ui);
+/// # });
 /// ```
 #[macro_export]
 macro_rules! trace {
-    ($ui:expr) => {{
+    ($ui: expr) => {{
         $ui.trace_location(format!("{}:{}", file!(), line!()))
     }};
-    ($ui:expr, $label:expr) => {{
+    ($ui: expr, $label: expr) => {{
         $ui.trace_location(format!("{} - {}:{}", $label, file!(), line!()))
     }};
 }
 
 // ----------------------------------------------------------------------------
 
-/// An assert that is only active when `egui` is compiled with the `egui_assert` feature
-/// or with the `debug_egui_assert` feature in debug builds.
+/// An assert that is only active when `egui` is compiled with the `extra_asserts` feature
+/// or with the `extra_debug_asserts` feature in debug builds.
 #[macro_export]
 macro_rules! egui_assert {
-    ($($arg:tt)*) => {
+    ($($arg: tt)*) => {
         if cfg!(any(
             feature = "extra_asserts",
             all(feature = "extra_debug_asserts", debug_assertions),
@@ -504,6 +549,7 @@ pub mod special_emojis {
 
 /// The different types of built-in widgets in egui
 #[derive(Clone, Copy, Debug, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub enum WidgetType {
     Label, // TODO: emit Label events
     Hyperlink,
@@ -524,3 +570,23 @@ pub enum WidgetType {
     /// If this is something you think should be added, file an issue.
     Other,
 }
+
+// ----------------------------------------------------------------------------
+
+/// For use in tests; especially doctests.
+pub fn __run_test_ctx(mut run_ui: impl FnMut(&CtxRef)) {
+    let mut ctx = CtxRef::default();
+    let _ = ctx.run(Default::default(), |ctx| {
+        run_ui(ctx);
+    });
+}
+
+/// For use in tests; especially doctests.
+pub fn __run_test_ui(mut add_contents: impl FnMut(&mut Ui)) {
+    let mut ctx = CtxRef::default();
+    let _ = ctx.run(Default::default(), |ctx| {
+        crate::CentralPanel::default().show(ctx, |ui| {
+            add_contents(ui);
+        });
+    });
+}
