@@ -1,34 +1,47 @@
 //! Simple plotting library.
 
+use crate::*;
+use epaint::ahash::AHashSet;
+use epaint::color::Hsva;
+use epaint::util::FloatOrd;
+use items::PlotItem;
+use legend::LegendWidget;
+use transform::{PlotBounds, ScreenTransform};
+
+pub use items::{
+    Arrows, Bar, BarChart, BoxElem, BoxPlot, BoxSpread, HLine, Line, LineStyle, MarkerShape,
+    PlotImage, Points, Polygon, Text, VLine, Value, Values,
+};
+pub use legend::{Corner, Legend};
+
 mod items;
 mod legend;
 mod transform;
 
-use std::collections::HashSet;
-
-use items::PlotItem;
-pub use items::{
-    Arrows, HLine, Line, LineStyle, MarkerShape, PlotImage, Points, Polygon, Text, VLine, Value,
-    Values,
-};
-use legend::LegendWidget;
-pub use legend::{Corner, Legend};
-use transform::{Bounds, ScreenTransform};
-
-use crate::*;
-use color::Hsva;
+type CustomLabelFunc = dyn Fn(&str, &Value) -> String;
+type CustomLabelFuncRef = Option<Box<CustomLabelFunc>>;
 
 // ----------------------------------------------------------------------------
 
 /// Information about the plot that has to persist between frames.
-#[cfg_attr(feature = "persistence", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 #[derive(Clone)]
 struct PlotMemory {
-    bounds: Bounds,
     auto_bounds: bool,
     hovered_entry: Option<String>,
-    hidden_items: HashSet<String>,
-    min_auto_bounds: Bounds,
+    hidden_items: AHashSet<String>,
+    min_auto_bounds: PlotBounds,
+    last_screen_transform: ScreenTransform,
+}
+
+impl PlotMemory {
+    pub fn load(ctx: &Context, id: Id) -> Option<Self> {
+        ctx.memory().data.get_persisted(id)
+    }
+
+    pub fn store(self, ctx: &Context, id: Id) {
+        ctx.memory().data.insert_persisted(id, self);
+    }
 }
 
 // ----------------------------------------------------------------------------
@@ -38,28 +51,24 @@ struct PlotMemory {
 /// `Plot` supports multiple lines and points.
 ///
 /// ```
-/// # let ui = &mut egui::Ui::__test();
+/// # egui::__run_test_ui(|ui| {
 /// use egui::plot::{Line, Plot, Value, Values};
 /// let sin = (0..1000).map(|i| {
 ///     let x = i as f64 * 0.01;
 ///     Value::new(x, x.sin())
 /// });
 /// let line = Line::new(Values::from_values_iter(sin));
-/// ui.add(
-///     Plot::new("my_plot").line(line).view_aspect(2.0)
-/// );
+/// Plot::new("my_plot").view_aspect(2.0).show(ui, |plot_ui| plot_ui.line(line));
+/// # });
 /// ```
 pub struct Plot {
     id_source: Id,
-    next_auto_color_idx: usize,
-
-    items: Vec<Box<dyn PlotItem>>,
 
     center_x_axis: bool,
     center_y_axis: bool,
     allow_zoom: bool,
     allow_drag: bool,
-    min_auto_bounds: Bounds,
+    min_auto_bounds: PlotBounds,
     margin_fraction: Vec2,
 
     min_size: Vec2,
@@ -70,6 +79,7 @@ pub struct Plot {
 
     show_x: bool,
     show_y: bool,
+    custom_label_func: CustomLabelFuncRef,
     legend_config: Option<Legend>,
     show_background: bool,
     show_axes: [bool; 2],
@@ -80,15 +90,12 @@ impl Plot {
     pub fn new(id_source: impl std::hash::Hash) -> Self {
         Self {
             id_source: Id::new(id_source),
-            next_auto_color_idx: 0,
-
-            items: Default::default(),
 
             center_x_axis: false,
             center_y_axis: false,
             allow_zoom: true,
             allow_drag: true,
-            min_auto_bounds: Bounds::NOTHING,
+            min_auto_bounds: PlotBounds::NOTHING,
             margin_fraction: Vec2::splat(0.05),
 
             min_size: Vec2::splat(64.0),
@@ -99,114 +106,13 @@ impl Plot {
 
             show_x: true,
             show_y: true,
+            custom_label_func: None,
             legend_config: None,
             show_background: true,
             show_axes: [true; 2],
         }
     }
 
-    fn auto_color(&mut self) -> Color32 {
-        let i = self.next_auto_color_idx;
-        self.next_auto_color_idx += 1;
-        let golden_ratio = (5.0_f32.sqrt() - 1.0) / 2.0; // 0.61803398875
-        let h = i as f32 * golden_ratio;
-        Hsva::new(h, 0.85, 0.5, 1.0).into() // TODO: OkLab or some other perspective color space
-    }
-
-    /// Add a data lines.
-    pub fn line(mut self, mut line: Line) -> Self {
-        if line.series.is_empty() {
-            return self;
-        };
-
-        // Give the stroke an automatic color if no color has been assigned.
-        if line.stroke.color == Color32::TRANSPARENT {
-            line.stroke.color = self.auto_color();
-        }
-        self.items.push(Box::new(line));
-        self
-    }
-
-    /// Add a polygon. The polygon has to be convex.
-    pub fn polygon(mut self, mut polygon: Polygon) -> Self {
-        if polygon.series.is_empty() {
-            return self;
-        };
-
-        // Give the stroke an automatic color if no color has been assigned.
-        if polygon.stroke.color == Color32::TRANSPARENT {
-            polygon.stroke.color = self.auto_color();
-        }
-        self.items.push(Box::new(polygon));
-        self
-    }
-
-    /// Add a text.
-    pub fn text(mut self, text: Text) -> Self {
-        if text.text.is_empty() {
-            return self;
-        };
-
-        self.items.push(Box::new(text));
-        self
-    }
-
-    /// Add data points.
-    pub fn points(mut self, mut points: Points) -> Self {
-        if points.series.is_empty() {
-            return self;
-        };
-
-        // Give the points an automatic color if no color has been assigned.
-        if points.color == Color32::TRANSPARENT {
-            points.color = self.auto_color();
-        }
-        self.items.push(Box::new(points));
-        self
-    }
-
-    /// Add arrows.
-    pub fn arrows(mut self, mut arrows: Arrows) -> Self {
-        if arrows.origins.is_empty() || arrows.tips.is_empty() {
-            return self;
-        };
-
-        // Give the arrows an automatic color if no color has been assigned.
-        if arrows.color == Color32::TRANSPARENT {
-            arrows.color = self.auto_color();
-        }
-        self.items.push(Box::new(arrows));
-        self
-    }
-
-    /// Add an image.
-    pub fn image(mut self, image: PlotImage) -> Self {
-        self.items.push(Box::new(image));
-        self
-    }
-
-    /// Add a horizontal line.
-    /// Can be useful e.g. to show min/max bounds or similar.
-    /// Always fills the full width of the plot.
-    pub fn hline(mut self, mut hline: HLine) -> Self {
-        if hline.stroke.color == Color32::TRANSPARENT {
-            hline.stroke.color = self.auto_color();
-        }
-        self.items.push(Box::new(hline));
-        self
-    }
-
-    /// Add a vertical line.
-    /// Can be useful e.g. to show min/max bounds or similar.
-    /// Always fills the full height of the plot.
-    pub fn vline(mut self, mut vline: VLine) -> Self {
-        if vline.stroke.color == Color32::TRANSPARENT {
-            vline.stroke.color = self.auto_color();
-        }
-        self.items.push(Box::new(vline));
-        self
-    }
-
     /// width / height ratio of the data.
     /// For instance, it can be useful to set this to `1.0` for when the two axes show the same
     /// unit.
@@ -257,18 +163,6 @@ impl Plot {
         self
     }
 
-    #[deprecated = "Renamed center_x_axis"]
-    pub fn symmetrical_x_axis(mut self, on: bool) -> Self {
-        self.center_x_axis = on;
-        self
-    }
-
-    #[deprecated = "Renamed center_y_axis"]
-    pub fn symmetrical_y_axis(mut self, on: bool) -> Self {
-        self.center_y_axis = on;
-        self
-    }
-
     /// Always keep the x-axis centered. Default: `false`.
     pub fn center_x_axis(mut self, on: bool) -> Self {
         self.center_x_axis = on;
@@ -293,6 +187,35 @@ impl Plot {
         self
     }
 
+    /// Provide a function to customize the on-hovel label for the x and y axis
+    ///
+    /// ```
+    /// # egui::__run_test_ui(|ui| {
+    /// use egui::plot::{Line, Plot, Value, Values};
+    /// let sin = (0..1000).map(|i| {
+    ///     let x = i as f64 * 0.01;
+    ///     Value::new(x, x.sin())
+    /// });
+    /// let line = Line::new(Values::from_values_iter(sin));
+    /// Plot::new("my_plot").view_aspect(2.0)
+    /// .custom_label_func(|name, value| {
+    ///     if !name.is_empty() {
+    ///         format!("{}: {:.*}%", name, 1, value.y).to_string()
+    ///     } else {
+    ///         "".to_string()
+    ///     }
+    /// })
+    /// .show(ui, |plot_ui| plot_ui.line(line));
+    /// # });
+    /// ```
+    pub fn custom_label_func<F: 'static + Fn(&str, &Value) -> String>(
+        mut self,
+        custom_lebel_func: F,
+    ) -> Self {
+        self.custom_label_func = Some(Box::new(custom_lebel_func));
+        self
+    }
+
     /// Expand bounds to include the given x value.
     /// For instance, to always show the y axis, call `plot.include_x(0.0)`.
     pub fn include_x(mut self, x: impl Into<f64>) -> Self {
@@ -307,13 +230,6 @@ impl Plot {
         self
     }
 
-    #[deprecated = "Use `Plot::legend` instead"]
-    /// Whether to show a legend including all named items. Default: `true`.
-    pub fn show_legend(mut self, show: bool) -> Self {
-        self.legend_config = show.then(Legend::default);
-        self
-    }
-
     /// Show a legend including all named items.
     pub fn legend(mut self, legend: Legend) -> Self {
         self.legend_config = Some(legend);
@@ -335,14 +251,11 @@ impl Plot {
         self.show_axes = show;
         self
     }
-}
 
-impl Widget for Plot {
-    fn ui(self, ui: &mut Ui) -> Response {
+    /// Interact with and add items to the plot and finally draw it.
+    pub fn show<R>(self, ui: &mut Ui, build_fn: impl FnOnce(&mut PlotUi) -> R) -> InnerResponse<R> {
         let Self {
             id_source,
-            next_auto_color_idx: _,
-            mut items,
             center_x_axis,
             center_y_axis,
             allow_zoom,
@@ -356,44 +269,12 @@ impl Widget for Plot {
             view_aspect,
             mut show_x,
             mut show_y,
+            custom_label_func,
             legend_config,
             show_background,
             show_axes,
         } = self;
 
-        let plot_id = ui.make_persistent_id(id_source);
-        let mut memory = ui
-            .memory()
-            .id_data
-            .get_mut_or_insert_with(plot_id, || PlotMemory {
-                bounds: min_auto_bounds,
-                auto_bounds: !min_auto_bounds.is_valid(),
-                hovered_entry: None,
-                hidden_items: HashSet::new(),
-                min_auto_bounds,
-            })
-            .clone();
-
-        // If the min bounds changed, recalculate everything.
-        if min_auto_bounds != memory.min_auto_bounds {
-            memory = PlotMemory {
-                bounds: min_auto_bounds,
-                auto_bounds: !min_auto_bounds.is_valid(),
-                hovered_entry: None,
-                min_auto_bounds,
-                ..memory
-            };
-            ui.memory().id_data.insert(plot_id, memory.clone());
-        }
-
-        let PlotMemory {
-            mut bounds,
-            mut auto_bounds,
-            mut hovered_entry,
-            mut hidden_items,
-            ..
-        } = memory;
-
         // Determine the size of the plot in the UI
         let size = {
             let width = width
@@ -401,7 +282,7 @@ impl Widget for Plot {
                     if let (Some(height), Some(aspect)) = (height, view_aspect) {
                         height * aspect
                     } else {
-                        ui.available_size_before_wrap_finite().x
+                        ui.available_size_before_wrap().x
                     }
                 })
                 .at_least(min_size.x);
@@ -411,19 +292,69 @@ impl Widget for Plot {
                     if let Some(aspect) = view_aspect {
                         width / aspect
                     } else {
-                        ui.available_size_before_wrap_finite().y
+                        ui.available_size_before_wrap().y
                     }
                 })
                 .at_least(min_size.y);
             vec2(width, height)
         };
 
+        // Allocate the space.
         let (rect, response) = ui.allocate_exact_size(size, Sense::drag());
-        let plot_painter = ui.painter().sub_region(rect);
+
+        // Load or initialize the memory.
+        let plot_id = ui.make_persistent_id(id_source);
+        let mut memory = PlotMemory::load(ui.ctx(), plot_id).unwrap_or_else(|| PlotMemory {
+            auto_bounds: !min_auto_bounds.is_valid(),
+            hovered_entry: None,
+            hidden_items: Default::default(),
+            min_auto_bounds,
+            last_screen_transform: ScreenTransform::new(
+                rect,
+                min_auto_bounds,
+                center_x_axis,
+                center_y_axis,
+            ),
+        });
+
+        // If the min bounds changed, recalculate everything.
+        if min_auto_bounds != memory.min_auto_bounds {
+            memory = PlotMemory {
+                auto_bounds: !min_auto_bounds.is_valid(),
+                hovered_entry: None,
+                min_auto_bounds,
+                ..memory
+            };
+            memory.clone().store(ui.ctx(), plot_id);
+        }
+
+        let PlotMemory {
+            mut auto_bounds,
+            mut hovered_entry,
+            mut hidden_items,
+            last_screen_transform,
+            ..
+        } = memory;
+
+        // Call the plot build function.
+        let mut plot_ui = PlotUi {
+            items: Vec::new(),
+            next_auto_color_idx: 0,
+            last_screen_transform,
+            response,
+            ctx: ui.ctx().clone(),
+        };
+        let inner = build_fn(&mut plot_ui);
+        let PlotUi {
+            mut items,
+            mut response,
+            last_screen_transform,
+            ..
+        } = plot_ui;
 
         // Background
         if show_background {
-            plot_painter.add(Shape::Rect {
+            ui.painter().sub_region(rect).add(epaint::RectShape {
                 rect,
                 corner_radius: 2.0,
                 fill: ui.visuals().extreme_bg_color,
@@ -431,10 +362,9 @@ impl Widget for Plot {
             });
         }
 
-        // Legend
+        // --- Legend ---
         let legend = legend_config
             .and_then(|config| LegendWidget::try_new(rect, config, &items, &hidden_items));
-
         // Don't show hover cursor when hovering over legend.
         if hovered_entry.is_some() {
             show_x = false;
@@ -452,6 +382,10 @@ impl Widget for Plot {
         // Move highlighted items to front.
         items.sort_by_key(|item| item.highlighted());
 
+        // --- Bound computation ---
+        let mut bounds = *last_screen_transform.bounds();
+
+        // Allow double clicking to reset to automatic bounds.
         auto_bounds |= response.double_clicked_by(PointerButton::Primary);
 
         // Set bounds automatically based on content.
@@ -462,18 +396,6 @@ impl Widget for Plot {
                 .for_each(|item| bounds.merge(&item.get_bounds()));
             bounds.add_relative_margin(margin_fraction);
         }
-        // Make sure they are not empty.
-        if !bounds.is_valid() {
-            bounds = Bounds::new_symmetrical(1.0);
-        }
-
-        // Scale axes so that the origin is in the center.
-        if center_x_axis {
-            bounds.make_x_symmetrical();
-        };
-        if center_y_axis {
-            bounds.make_y_symmetrical()
-        };
 
         let mut transform = ScreenTransform::new(rect, bounds, center_x_axis, center_y_axis);
 
@@ -484,6 +406,7 @@ impl Widget for Plot {
 
         // Dragging
         if allow_drag && response.dragged_by(PointerButton::Primary) {
+            response = response.on_hover_cursor(CursorIcon::Grabbing);
             transform.translate_bounds(-response.drag_delta());
             auto_bounds = false;
         }
@@ -514,14 +437,13 @@ impl Widget for Plot {
             .iter_mut()
             .for_each(|item| item.initialize(transform.bounds().range_x()));
 
-        let bounds = *transform.bounds();
-
-        let prepared = Prepared {
+        let prepared = PreparedPlot {
             items,
             show_x,
             show_y,
+            custom_label_func,
             show_axes,
-            transform,
+            transform: transform.clone(),
         };
         prepared.ui(ui, &response);
 
@@ -531,34 +453,208 @@ impl Widget for Plot {
             hovered_entry = legend.get_hovered_entry_name();
         }
 
-        ui.memory().id_data.insert(
-            plot_id,
-            PlotMemory {
-                bounds,
-                auto_bounds,
-                hovered_entry,
-                hidden_items,
-                min_auto_bounds,
-            },
-        );
+        let memory = PlotMemory {
+            auto_bounds,
+            hovered_entry,
+            hidden_items,
+            min_auto_bounds,
+            last_screen_transform: transform,
+        };
+        memory.store(ui.ctx(), plot_id);
 
-        if show_x || show_y {
+        let response = if show_x || show_y {
             response.on_hover_cursor(CursorIcon::Crosshair)
         } else {
             response
-        }
+        };
+
+        InnerResponse { inner, response }
     }
 }
 
-struct Prepared {
+/// Provides methods to interact with a plot while building it. It is the single argument of the closure
+/// provided to `Plot::show`. See [`Plot`] for an example of how to use it.
+pub struct PlotUi {
+    items: Vec<Box<dyn PlotItem>>,
+    next_auto_color_idx: usize,
+    last_screen_transform: ScreenTransform,
+    response: Response,
+    ctx: CtxRef,
+}
+
+impl PlotUi {
+    fn auto_color(&mut self) -> Color32 {
+        let i = self.next_auto_color_idx;
+        self.next_auto_color_idx += 1;
+        let golden_ratio = (5.0_f32.sqrt() - 1.0) / 2.0; // 0.61803398875
+        let h = i as f32 * golden_ratio;
+        Hsva::new(h, 0.85, 0.5, 1.0).into() // TODO: OkLab or some other perspective color space
+    }
+
+    pub fn ctx(&self) -> &CtxRef {
+        &self.ctx
+    }
+
+    /// The plot bounds as they were in the last frame. If called on the first frame and the bounds were not
+    /// further specified in the plot builder, this will return bounds centered on the origin. The bounds do
+    /// not change until the plot is drawn.
+    pub fn plot_bounds(&self) -> PlotBounds {
+        *self.last_screen_transform.bounds()
+    }
+
+    /// Returns `true` if the plot area is currently hovered.
+    pub fn plot_hovered(&self) -> bool {
+        self.response.hovered()
+    }
+
+    /// The pointer position in plot coordinates. Independent of whether the pointer is in the plot area.
+    pub fn pointer_coordinate(&self) -> Option<Value> {
+        // We need to subtract the drag delta to keep in sync with the frame-delayed screen transform:
+        let last_pos = self.ctx().input().pointer.latest_pos()? - self.response.drag_delta();
+        let value = self.plot_from_screen(last_pos);
+        Some(value)
+    }
+
+    /// The pointer drag delta in plot coordinates.
+    pub fn pointer_coordinate_drag_delta(&self) -> Vec2 {
+        let delta = self.response.drag_delta();
+        let dp_dv = self.last_screen_transform.dpos_dvalue();
+        Vec2::new(delta.x / dp_dv[0] as f32, delta.y / dp_dv[1] as f32)
+    }
+
+    /// Transform the plot coordinates to screen coordinates.
+    pub fn screen_from_plot(&self, position: Value) -> Pos2 {
+        self.last_screen_transform.position_from_value(&position)
+    }
+
+    /// Transform the screen coordinates to plot coordinates.
+    pub fn plot_from_screen(&self, position: Pos2) -> Value {
+        self.last_screen_transform.value_from_position(position)
+    }
+
+    /// Add a data line.
+    pub fn line(&mut self, mut line: Line) {
+        if line.series.is_empty() {
+            return;
+        };
+
+        // Give the stroke an automatic color if no color has been assigned.
+        if line.stroke.color == Color32::TRANSPARENT {
+            line.stroke.color = self.auto_color();
+        }
+        self.items.push(Box::new(line));
+    }
+
+    /// Add a polygon. The polygon has to be convex.
+    pub fn polygon(&mut self, mut polygon: Polygon) {
+        if polygon.series.is_empty() {
+            return;
+        };
+
+        // Give the stroke an automatic color if no color has been assigned.
+        if polygon.stroke.color == Color32::TRANSPARENT {
+            polygon.stroke.color = self.auto_color();
+        }
+        self.items.push(Box::new(polygon));
+    }
+
+    /// Add a text.
+    pub fn text(&mut self, text: Text) {
+        if text.text.is_empty() {
+            return;
+        };
+
+        self.items.push(Box::new(text));
+    }
+
+    /// Add data points.
+    pub fn points(&mut self, mut points: Points) {
+        if points.series.is_empty() {
+            return;
+        };
+
+        // Give the points an automatic color if no color has been assigned.
+        if points.color == Color32::TRANSPARENT {
+            points.color = self.auto_color();
+        }
+        self.items.push(Box::new(points));
+    }
+
+    /// Add arrows.
+    pub fn arrows(&mut self, mut arrows: Arrows) {
+        if arrows.origins.is_empty() || arrows.tips.is_empty() {
+            return;
+        };
+
+        // Give the arrows an automatic color if no color has been assigned.
+        if arrows.color == Color32::TRANSPARENT {
+            arrows.color = self.auto_color();
+        }
+        self.items.push(Box::new(arrows));
+    }
+
+    /// Add an image.
+    pub fn image(&mut self, image: PlotImage) {
+        self.items.push(Box::new(image));
+    }
+
+    /// Add a horizontal line.
+    /// Can be useful e.g. to show min/max bounds or similar.
+    /// Always fills the full width of the plot.
+    pub fn hline(&mut self, mut hline: HLine) {
+        if hline.stroke.color == Color32::TRANSPARENT {
+            hline.stroke.color = self.auto_color();
+        }
+        self.items.push(Box::new(hline));
+    }
+
+    /// Add a vertical line.
+    /// Can be useful e.g. to show min/max bounds or similar.
+    /// Always fills the full height of the plot.
+    pub fn vline(&mut self, mut vline: VLine) {
+        if vline.stroke.color == Color32::TRANSPARENT {
+            vline.stroke.color = self.auto_color();
+        }
+        self.items.push(Box::new(vline));
+    }
+
+    /// Add a box plot diagram.
+    pub fn box_plot(&mut self, mut box_plot: BoxPlot) {
+        if box_plot.boxes.is_empty() {
+            return;
+        }
+
+        // Give the elements an automatic color if no color has been assigned.
+        if box_plot.default_color == Color32::TRANSPARENT {
+            box_plot = box_plot.color(self.auto_color());
+        }
+        self.items.push(Box::new(box_plot));
+    }
+
+    /// Add a bar chart.
+    pub fn bar_chart(&mut self, mut chart: BarChart) {
+        if chart.bars.is_empty() {
+            return;
+        }
+
+        // Give the elements an automatic color if no color has been assigned.
+        if chart.default_color == Color32::TRANSPARENT {
+            chart = chart.color(self.auto_color());
+        }
+        self.items.push(Box::new(chart));
+    }
+}
+
+struct PreparedPlot {
     items: Vec<Box<dyn PlotItem>>,
     show_x: bool,
     show_y: bool,
+    custom_label_func: CustomLabelFuncRef,
     show_axes: [bool; 2],
     transform: ScreenTransform,
 }
 
-impl Prepared {
+impl PreparedPlot {
     fn ui(self, ui: &mut Ui, response: &Response) {
         let mut shapes = Vec::new();
 
@@ -645,21 +741,16 @@ impl Prepared {
                 let color = color_from_alpha(ui, text_alpha);
                 let text = emath::round_to_decimals(value_main, 5).to_string(); // hack
 
-                let galley = ui.fonts().layout_single_line(text_style, text);
+                let galley = ui.painter().layout_no_wrap(text, text_style, color);
 
-                let mut text_pos = pos_in_gui + vec2(1.0, -galley.size.y);
+                let mut text_pos = pos_in_gui + vec2(1.0, -galley.size().y);
 
                 // Make sure we see the labels, even if the axis is off-screen:
                 text_pos[1 - axis] = text_pos[1 - axis]
-                    .at_most(transform.frame().max[1 - axis] - galley.size[1 - axis] - 2.0)
+                    .at_most(transform.frame().max[1 - axis] - galley.size()[1 - axis] - 2.0)
                     .at_least(transform.frame().min[1 - axis] + 1.0);
 
-                shapes.push(Shape::Text {
-                    pos: text_pos,
-                    galley,
-                    color,
-                    fake_italics: false,
-                });
+                shapes.push(Shape::galley(text_pos, galley));
             }
         }
 
@@ -677,6 +768,7 @@ impl Prepared {
             transform,
             show_x,
             show_y,
+            custom_label_func,
             items,
             ..
         } = self;
@@ -685,88 +777,31 @@ impl Prepared {
             return;
         }
 
-        let interact_radius: f32 = 16.0;
-        let mut closest_value = None;
-        let mut closest_item = None;
-        let mut closest_dist_sq = interact_radius.powi(2);
-        for item in items {
-            if let Some(values) = item.values() {
-                for value in &values.values {
-                    let pos = transform.position_from_value(value);
-                    let dist_sq = pointer.distance_sq(pos);
-                    if dist_sq < closest_dist_sq {
-                        closest_dist_sq = dist_sq;
-                        closest_value = Some(value);
-                        closest_item = Some(item.name());
-                    }
-                }
-            }
-        }
+        let interact_radius_sq: f32 = (16.0f32).powi(2);
 
-        let mut prefix = String::new();
-        if let Some(name) = closest_item {
-            if !name.is_empty() {
-                prefix = format!("{}\n", name);
-            }
-        }
+        let candidates = items.iter().filter_map(|item| {
+            let item = &**item;
+            let closest = item.find_closest(pointer, transform);
 
-        let line_color = if ui.visuals().dark_mode {
-            Color32::from_gray(100).additive()
+            Some(item).zip(closest)
+        });
+
+        let closest = candidates
+            .min_by_key(|(_, elem)| elem.dist_sq.ord())
+            .filter(|(_, elem)| elem.dist_sq <= interact_radius_sq);
+
+        let plot = items::PlotConfig {
+            ui,
+            transform,
+            show_x: *show_x,
+            show_y: *show_y,
+        };
+
+        if let Some((item, elem)) = closest {
+            item.on_hover(elem, shapes, &plot, custom_label_func);
         } else {
-            Color32::from_black_alpha(180)
-        };
-
-        let value = if let Some(value) = closest_value {
-            let position = transform.position_from_value(value);
-            shapes.push(Shape::circle_filled(position, 3.0, line_color));
-            *value
-        } else {
-            transform.value_from_position(pointer)
-        };
-        let pointer = transform.position_from_value(&value);
-
-        let rect = transform.frame();
-
-        if *show_x {
-            // vertical line
-            shapes.push(Shape::line_segment(
-                [pos2(pointer.x, rect.top()), pos2(pointer.x, rect.bottom())],
-                (1.0, line_color),
-            ));
+            let value = transform.value_from_position(pointer);
+            items::rulers_at_value(pointer, value, "", &plot, shapes, custom_label_func);
         }
-        if *show_y {
-            // horizontal line
-            shapes.push(Shape::line_segment(
-                [pos2(rect.left(), pointer.y), pos2(rect.right(), pointer.y)],
-                (1.0, line_color),
-            ));
-        }
-
-        let text = {
-            let scale = transform.dvalue_dpos();
-            let x_decimals = ((-scale[0].abs().log10()).ceil().at_least(0.0) as usize).at_most(6);
-            let y_decimals = ((-scale[1].abs().log10()).ceil().at_least(0.0) as usize).at_most(6);
-            if *show_x && *show_y {
-                format!(
-                    "{}x = {:.*}\ny = {:.*}",
-                    prefix, x_decimals, value.x, y_decimals, value.y
-                )
-            } else if *show_x {
-                format!("{}x = {:.*}", prefix, x_decimals, value.x)
-            } else if *show_y {
-                format!("{}y = {:.*}", prefix, y_decimals, value.y)
-            } else {
-                unreachable!()
-            }
-        };
-
-        shapes.push(Shape::text(
-            ui.fonts(),
-            pointer + vec2(3.0, -2.0),
-            Align2::LEFT_BOTTOM,
-            text,
-            TextStyle::Body,
-            ui.visuals().text_color(),
-        ));
     }
 }
